bl_info = {
    "name": "3DM Addon",
    "description": "Nodes Groups, Render Panel, Light setup",
    "author": "Nathan Boder",
    "version": (2, 2),
    "blender": (2, 90, 0),
    "location": "Node Editor > Add, Node Editor > Tools, Propertiers Panel Render, Propertiers Panel Light and Viewport > Tools",
    "category": "Nodes groups"
}
    
# Libraries
import bpy, os
from bpy.types import PropertyGroup

# Shader Nodes Group
############################################################################################################

# Add the 3DM Material   
class ThirdDimensionMediaPanel_OT_Add3dmMaterial(bpy.types.Operator):
    bl_idname = "thirddimensionmedia.add3dmmaterial"
    bl_label = "3DM Material"
    bl_description = "Add 3DM Material Nodes Group"
    
    @classmethod
    def poll(cls, context):
        space = context.space_data

        # Check that it's a ShaderNodeTree, editing 'OBJECT' type and that a node tree (with nodes) exists
        return (space.type == 'NODE_EDITOR') and (space.tree_type == 'ShaderNodeTree') and (space.shader_type == 'OBJECT') and (hasattr(space,"node_tree")) and (hasattr(space.node_tree,"nodes"))

    def execute(self, context):
        if "3DM_Node_Material_v2" in bpy.data.node_groups:
            bpy.ops.node.add_node(type="ShaderNodeGroup", settings=[{"name":"node_tree","value":"bpy.data.node_groups['3DM_Node_Material_v2']"}], use_transform=True)
            bpy.ops.node.translate_attach_remove_on_cancel('INVOKE_DEFAULT')
        else:
            path = os.path.join(os.path.dirname(__file__), "blends/thirdgroups.blend/NodeTree/")
            bpy.ops.wm.append( filename = '3DM_Node_Material_v2', directory = path)
            bpy.ops.node.add_node(type="ShaderNodeGroup", settings=[{"name":"node_tree","value":"bpy.data.node_groups['3DM_Node_Material_v2']"}], use_transform=True)
            bpy.ops.node.translate_attach_remove_on_cancel('INVOKE_DEFAULT')
            
        return{'FINISHED'}

# Add the 3DM Glass       
class ThirdDimensionMediaPanel_OT_Add3dmGlass(bpy.types.Operator):
    bl_idname = "thirddimensionmedia.add3dmglass"
    bl_label = "3DM Glass"
    bl_description = "Add 3DM Glass Nodes Group"
    
    @classmethod
    def poll(cls, context):
        space = context.space_data

        # Check that it's a ShaderNodeTree, editing 'OBJECT' type and that a node tree (with nodes) exists
        return (space.type == 'NODE_EDITOR') and (space.tree_type == 'ShaderNodeTree') and (space.shader_type == 'OBJECT') and (hasattr(space,"node_tree")) and (hasattr(space.node_tree,"nodes"))

    def execute(self, context):
        if "3DM_Glass" in bpy.data.node_groups:
            bpy.ops.node.add_node(type="ShaderNodeGroup", settings=[{"name":"node_tree","value":"bpy.data.node_groups['3DM_Glass']"}], use_transform=True)
            bpy.ops.node.translate_attach_remove_on_cancel('INVOKE_DEFAULT')
        else:
            path = os.path.join(os.path.dirname(__file__), "blends/thirdgroups.blend/NodeTree/")
            bpy.ops.wm.append( filename = '3DM_Glass', directory = path)
            bpy.ops.node.add_node(type="ShaderNodeGroup", settings=[{"name":"node_tree","value":"bpy.data.node_groups['3DM_Glass']"}], use_transform=True)
            bpy.ops.node.translate_attach_remove_on_cancel('INVOKE_DEFAULT')
            
        return{'FINISHED'}
    
# Add the 3DM Leaf Material      
class ThirdDimensionMediaPanel_OT_Add3dmLeaf(bpy.types.Operator):
    bl_idname = "thirddimensionmedia.add3dmleaf"
    bl_label = "3DM Leaf"
    bl_description = "Add 3DM Leaf Nodes Group"
    
    @classmethod
    def poll(cls, context):
        space = context.space_data

        # Check that it's a ShaderNodeTree, editing 'OBJECT' type and that a node tree (with nodes) exists
        return (space.type == 'NODE_EDITOR') and (space.tree_type == 'ShaderNodeTree') and (space.shader_type == 'OBJECT') and (hasattr(space,"node_tree")) and (hasattr(space.node_tree,"nodes"))

    def execute(self, context):
        if "3DM_Node_Leaf_v2" in bpy.data.node_groups:
            bpy.ops.node.add_node(type="ShaderNodeGroup", settings=[{"name":"node_tree","value":"bpy.data.node_groups['3DM_Node_Leaf_v2']"}], use_transform=True)
            bpy.ops.node.translate_attach_remove_on_cancel('INVOKE_DEFAULT')
        else:
            path = os.path.join(os.path.dirname(__file__), "blends/thirdgroups.blend/NodeTree/")
            bpy.ops.wm.append( filename = '3DM_Node_Leaf_v2', directory = path)
            bpy.ops.node.add_node(type="ShaderNodeGroup", settings=[{"name":"node_tree","value":"bpy.data.node_groups['3DM_Node_Leaf_v2']"}], use_transform=True)
            bpy.ops.node.translate_attach_remove_on_cancel('INVOKE_DEFAULT')
            
        return{'FINISHED'}

# Add the 3DM Textures Preferencies    
class ThirdDimensionMediaPanel_OT_Add3dmTprefs(bpy.types.Operator):
    bl_idname = "thirddimensionmedia.add3dmtprefs"
    bl_label = "3DM Textures Preferencies"
    bl_description = "Add 3DM Textures Preferencies Nodes Group"
    
    @classmethod
    def poll(cls, context):
        space = context.space_data

        # Check that it's a ShaderNodeTree, editing 'OBJECT' type and that a node tree (with nodes) exists
        return (space.type == 'NODE_EDITOR') and (space.tree_type == 'ShaderNodeTree') and (space.shader_type == 'OBJECT') and (hasattr(space,"node_tree")) and (hasattr(space.node_tree,"nodes"))

    def execute(self, context):
        if "3DM_Textures_Preferencies" in bpy.data.node_groups:
            bpy.ops.node.add_node(type="ShaderNodeGroup", settings=[{"name":"node_tree","value":"bpy.data.node_groups['3DM_Textures_Preferencies']"}], use_transform=True)
            bpy.ops.node.translate_attach_remove_on_cancel('INVOKE_DEFAULT')
        else:
            path = os.path.join(os.path.dirname(__file__), "blends/thirdgroups.blend/NodeTree/")
            bpy.ops.wm.append( filename = '3DM_Textures_Preferencies', directory = path)
            bpy.ops.node.add_node(type="ShaderNodeGroup", settings=[{"name":"node_tree","value":"bpy.data.node_groups['3DM_Textures_Preferencies']"}], use_transform=True)
            bpy.ops.node.translate_attach_remove_on_cancel('INVOKE_DEFAULT')
            
        return{'FINISHED'}

# Add the 3DM Metal Color    
class ThirdDimensionMediaPanel_OT_Add3dmMcolor(bpy.types.Operator):
    bl_idname = "thirddimensionmedia.add3dmmcolor"
    bl_label = "3DM Metal Color"
    bl_description = "Add 3DM Metal Color Nodes Group"
    
    @classmethod
    def poll(cls, context):
        space = context.space_data

        # Check that it's a ShaderNodeTree, editing 'OBJECT' type and that a node tree (with nodes) exists
        return (space.type == 'NODE_EDITOR') and (space.tree_type == 'ShaderNodeTree') and (space.shader_type == 'OBJECT') and (hasattr(space,"node_tree")) and (hasattr(space.node_tree,"nodes"))

    def execute(self, context):
        if "3DM_Metal_Color" in bpy.data.node_groups:
            bpy.ops.node.add_node(type="ShaderNodeGroup", settings=[{"name":"node_tree","value":"bpy.data.node_groups['3DM_Metal_Color']"}], use_transform=True)
            bpy.ops.node.translate_attach_remove_on_cancel('INVOKE_DEFAULT')
        else:
            path = os.path.join(os.path.dirname(__file__), "blends/thirdgroups.blend/NodeTree/")
            bpy.ops.wm.append( filename = '3DM_Metal_Color', directory = path)
            bpy.ops.node.add_node(type="ShaderNodeGroup", settings=[{"name":"node_tree","value":"bpy.data.node_groups['3DM_Metal_Color']"}], use_transform=True)
            bpy.ops.node.translate_attach_remove_on_cancel('INVOKE_DEFAULT')
            
        return{'FINISHED'}

# Compositor Nodes Group
############################################################################################################

# Add the 3DM Postproduction     
class ThirdDimensionMediaPanel_OT_Add3dmPost(bpy.types.Operator):
    bl_idname = "thirddimensionmedia.add3dmpost"
    bl_label = "3DM Post Production"
    bl_description = "Add 3DM Post Production Nodes Group"
    
    @classmethod
    def poll(cls, context):
        space = context.space_data

        # Check that it's a ShaderNodeTree, editing 'OBJECT' type and that a node tree (with nodes) exists
        return (space.type == 'NODE_EDITOR') and (space.tree_type == 'CompositorNodeTree') and (hasattr(space,"node_tree")) and (hasattr(space.node_tree,"nodes"))

    def execute(self, context):
        if "3DM_Post_Production" in bpy.data.node_groups:
            bpy.ops.node.add_node(type="CompositorNodeGroup", settings=[{"name":"node_tree","value":"bpy.data.node_groups['3DM_Post_Production']"}], use_transform=True)
            bpy.ops.node.translate_attach_remove_on_cancel('INVOKE_DEFAULT')
        else:
            path = os.path.join(os.path.dirname(__file__), "blends/thirdgroups.blend/NodeTree/")
            bpy.ops.wm.append( filename = '3DM_Post_Production', directory = path)
            bpy.ops.node.add_node(type="CompositorNodeGroup", settings=[{"name":"node_tree","value":"bpy.data.node_groups['3DM_Post_Production']"}], use_transform=True)
            bpy.ops.node.translate_attach_remove_on_cancel('INVOKE_DEFAULT')

        return{'FINISHED'}
    
# Add the 3DM Denoiser    
class ThirdDimensionMediaPanel_OT_Add3dmDenoiser(bpy.types.Operator):
    bl_idname = "thirddimensionmedia.add3dmdenoiser"
    bl_label = "3DM Denoiser"
    bl_description = "Add 3DM Denoiser Nodes Group"
    
    @classmethod
    def poll(cls, context):
        space = context.space_data

        # Check that it's a ShaderNodeTree, editing 'OBJECT' type and that a node tree (with nodes) exists
        return (space.type == 'NODE_EDITOR') and (space.tree_type == 'CompositorNodeTree') and (hasattr(space,"node_tree")) and (hasattr(space.node_tree,"nodes"))

    def execute(self, context):
        if "3DM_Denoiser" in bpy.data.node_groups:
            bpy.ops.node.add_node(type="CompositorNodeGroup", settings=[{"name":"node_tree","value":"bpy.data.node_groups['3DM_Denoiser']"}], use_transform=True)
            bpy.ops.node.translate_attach_remove_on_cancel('INVOKE_DEFAULT')
        else:
            path = os.path.join(os.path.dirname(__file__), "blends/thirdgroups.blend/NodeTree/")
            bpy.ops.wm.append( filename = '3DM_Denoiser', directory = path)
            bpy.ops.node.add_node(type="CompositorNodeGroup", settings=[{"name":"node_tree","value":"bpy.data.node_groups['3DM_Denoiser']"}], use_transform=True)
            bpy.ops.node.translate_attach_remove_on_cancel('INVOKE_DEFAULT')

        return{'FINISHED'}
    
# Add the 3DM Z/AO/DiffCol Nodes Group    
class ThirdDimensionMediaPanel_OT_Add3dmZ(bpy.types.Operator):
    bl_idname = "thirddimensionmedia.add3dmz"
    bl_label = "3DM Z/AO/DiffCol"
    bl_description = "Add 3DM Z/AO/DiffCol Nodes Group"
    
    @classmethod
    def poll(cls, context):
        space = context.space_data

        # Check that it's a ShaderNodeTree, editing 'OBJECT' type and that a node tree (with nodes) exists
        return (space.type == 'NODE_EDITOR') and (space.tree_type == 'CompositorNodeTree') and (hasattr(space,"node_tree")) and (hasattr(space.node_tree,"nodes"))

    def execute(self, context):
        if "3DM_Z_AO_DiffCol" in bpy.data.node_groups:
            bpy.ops.node.add_node(type="CompositorNodeGroup", settings=[{"name":"node_tree","value":"bpy.data.node_groups['3DM_Z_AO_DiffCol']"}], use_transform=True)
            bpy.ops.node.translate_attach_remove_on_cancel('INVOKE_DEFAULT')
        else:
            path = os.path.join(os.path.dirname(__file__), "blends/thirdgroups.blend/NodeTree/")
            bpy.ops.wm.append( filename = '3DM_Z_AO_DiffCol', directory = path)
            bpy.ops.node.add_node(type="CompositorNodeGroup", settings=[{"name":"node_tree","value":"bpy.data.node_groups['3DM_Z_AO_DiffCol']"}], use_transform=True)
            bpy.ops.node.translate_attach_remove_on_cancel('INVOKE_DEFAULT')

        return{'FINISHED'}

#Add the 3DM Mask ID    
class ThirdDimensionMediaPanel_OT_Add3dmMaskID(bpy.types.Operator):
    bl_idname = "thirddimensionmedia.add3dmmaskid"
    bl_label = "3DM Mask ID"
    bl_description = "Add 3DM Mask ID Nodes Group"
    
    @classmethod
    def poll(cls, context):
        space = context.space_data

        # Check that it's a ShaderNodeTree, editing 'OBJECT' type and that a node tree (with nodes) exists
        return (space.type == 'NODE_EDITOR') and (space.tree_type == 'CompositorNodeTree') and (hasattr(space,"node_tree")) and (hasattr(space.node_tree,"nodes"))

    def execute(self, context):
        if "3DM_Mask_ID" in bpy.data.node_groups:
            bpy.ops.node.add_node(type="CompositorNodeGroup", settings=[{"name":"node_tree","value":"bpy.data.node_groups['3DM_Mask_ID']"}], use_transform=True)
            bpy.ops.node.translate_attach_remove_on_cancel('INVOKE_DEFAULT')
        else:
            path = os.path.join(os.path.dirname(__file__), "blends/thirdgroups.blend/NodeTree/")
            bpy.ops.wm.append( filename = '3DM_Mask_ID', directory = path)
            bpy.ops.node.add_node(type="CompositorNodeGroup", settings=[{"name":"node_tree","value":"bpy.data.node_groups['3DM_Mask_ID']"}], use_transform=True)
            bpy.ops.node.translate_attach_remove_on_cancel('INVOKE_DEFAULT')

        return{'FINISHED'}



# 3DM Nodes Panel
############################################################################################################

class ThirdDimensionMediaNodesPanel(bpy.types.Panel):
    bl_idname = "NODE_PT_NodesPanel"
    bl_space_type = 'NODE_EDITOR'
    bl_label = "3DM Nodes Groups"
    bl_region_type = "UI"
    bl_category = "3DM Nodes"
    
    def draw(self, context):
        layout = self.layout
        
        box = layout.box()
        box.label(icon= 'NODE_MATERIAL', text="Shader")
        row = box.row()
        row.operator("thirddimensionmedia.add3dmmaterial", text="3DM Material")
        row = box.row()
        row.operator("thirddimensionmedia.add3dmglass", text="3DM Glass")
        row = box.row()
        row.operator("thirddimensionmedia.add3dmleaf", text="3DM Leaf")
        row = box.row()
        row.operator("thirddimensionmedia.add3dmtprefs", text="3DM Textures Preferencies")
        row = box.row()
        row.operator("thirddimensionmedia.add3dmmcolor", text="3DM Metal Color")
        row = box.row()
        row.operator("thirddimensionmedia.plastic2", text="Plastic")
        row.operator("thirddimensionmedia.metal2", text="Metal")
        row.operator("thirddimensionmedia.full2", text="Full setup")
        
        box = layout.box()
        box.label(icon= 'NODE_COMPOSITING', text="Compositor")
        row = box.row()
        row.operator("thirddimensionmedia.add3dmpost", text="3DM Post Production")
        row = box.row()
        row.operator("thirddimensionmedia.add3dmdenoiser", text="3DM Denoiser")
        row = box.row()
        row.operator("thirddimensionmedia.add3dmz", text="3DM Z/AO/DiffCol")
        row = box.row()
        row.operator("thirddimensionmedia.add3dmmaskid", text="3DM Mask ID")
        
        
# Sub-menu for Shader and Compositor Nodes Groups inside the "Add"
############################################################################################################   
class ThirdDimensionMediaMenu(bpy.types.Menu):
    bl_idname = "thirddimensionmedia.menu"
    bl_label = "3DM Menu"

    def draw(self, context):
        layout = self.layout
        
        row = layout.row()
        row.label(icon= 'NODE_MATERIAL', text="Shader")
        row = layout.row()
        row.operator("thirddimensionmedia.add3dmmaterial", text="3DM Material")
        row = layout.row()
        row.operator("thirddimensionmedia.add3dmglass", text="3DM Glass")
        row = layout.row()
        row.operator("thirddimensionmedia.add3dmleaf", text="3DM Leaf")
        row = layout.row()
        row.operator("thirddimensionmedia.add3dmtprefs", text="3DM Textures Preferencies")
        row = layout.row()
        row.operator("thirddimensionmedia.add3dmmcolor", text="3DM Metal Color")
        
        row = layout.row()
        row.label(icon= 'NODE_COMPOSITING', text="Compositar")
        row = layout.row()
        row.operator("thirddimensionmedia.add3dmpost", text="3DM Post Production")
        row = layout.row()
        row.operator("thirddimensionmedia.add3dmdenoiser", text="3DM Denoiser")
        row = layout.row()
        row.operator("thirddimensionmedia.add3dmz", text="3DM Z/AO/DiffCol")
        row = layout.row()
        row.operator("thirddimensionmedia.add3dmmaskid", text="3DM Mask ID")

# Main-menu inside the "Add"         
############################################################################################################
        
def menu_draw(self, context):
    layout = self.layout
    
    row = layout.row()
    layout.row().separator()
    self.layout.menu("thirddimensionmedia.menu", icon= 'GROUP',text="3DM Groups")
        
        
# 3DM Shortcuts
############################################################################################################
    
#### 3DM Viewport Panel ####
class ThirdDimensionMediaObjectPanel(bpy.types.Panel):
    bl_space_type = 'VIEW_3D'
    bl_region_type = 'UI'
    bl_category = '3DM'
    bl_label = '3DM Shortcuts'
    
    def draw(self, context):
        layout = self.layout
        
        box = layout.box()
        box.label(icon= 'MOD_BOOLEAN', text="Boolean")
        row = box.row()
        row.operator("thirddimensionmedia.setitbool", text="Ready to be a Boolean")
        box.label(icon= 'MOD_BEVEL', text="Bevel")
        row = box.row()
        row.operator("thirddimensionmedia.addbevel3", text="3mm")
        row.operator("thirddimensionmedia.addbevel5", text="5mm")
        row.operator("thirddimensionmedia.addbevel10", text="10mm")
        box.label(icon= 'CAMERA_DATA', text="Camera")
        row = box.row()
        row.operator("thirddimensionmedia.addcamera", text="Add a ready 15mm camera")
        
# Setup Boolean
                
class ThirdDimensionMediaObjectPanel_OT_SetItBool(bpy.types.Operator):
    bl_idname = "thirddimensionmedia.setitbool"
    bl_label = "3DM Set it Bool"
    bl_description = "Set this selection in wire mode for viewport and hide the visibily for camera, diffuse, glossy, transmission, volume scatter and shadow. "
    
    def execute(self, context):

        selection = bpy.context.selected_objects

        for i in selection:

            i.display_type = 'WIRE'
            i.cycles_visibility.camera = False
            i.cycles_visibility.diffuse = False
            i.cycles_visibility.glossy = False
            i.cycles_visibility.glossy = False
            i.cycles_visibility.transmission = False
            i.cycles_visibility.scatter = False
            i.cycles_visibility.scatter = False
            i.cycles_visibility.shadow = False


        return{'FINISHED'}
    
# Add Bevel 3mm
                
class ThirdDimensionMediaObjectPanel_OT_AddBevel3(bpy.types.Operator):
    bl_idname = "thirddimensionmedia.addbevel3"
    bl_label = "3DM Add bevel 3mm"
    bl_description = "Add 3DM Bevel with 3mm of width or change the existing 3DM Bevel to 3mm"
    
    def execute(self, context):
        
        selection = bpy.context.selected_objects

        for i in selection:
            
            mod = i.modifiers.get("3DM Bevel")
            if mod is None:
                # otherwise add a modifier to selected object
                mod = i.modifiers.new("3DM Bevel", 'BEVEL')
            i.modifiers["3DM Bevel"].width = 0.003
            i.modifiers["3DM Bevel"].use_clamp_overlap = False
            i.modifiers["3DM Bevel"].offset_type = 'WIDTH'
            i.modifiers["3DM Bevel"].limit_method = 'ANGLE'
                
        return{'FINISHED'}
    
# Add Bevel 5mm
                
class ThirdDimensionMediaObjectPanel_OT_AddBevel5(bpy.types.Operator):
    bl_idname = "thirddimensionmedia.addbevel5"
    bl_label = "3DM Add bevel 5mm"
    bl_description = "Add 3DM Bevel with 5mm of width or change the existing 3DM Bevel to 5mm"
    
    def execute(self, context):
        
        selection = bpy.context.selected_objects

        for i in selection:
            
            mod = i.modifiers.get("3DM Bevel")
            if mod is None:
                # otherwise add a modifier to selected object
                mod = i.modifiers.new("3DM Bevel", 'BEVEL')
            i.modifiers["3DM Bevel"].width = 0.005
            i.modifiers["3DM Bevel"].use_clamp_overlap = False
            i.modifiers["3DM Bevel"].offset_type = 'WIDTH'
            i.modifiers["3DM Bevel"].limit_method = 'ANGLE'
                
        return{'FINISHED'}
    
# Add Bevel 10mm
                
class ThirdDimensionMediaObjectPanel_OT_AddBevel10(bpy.types.Operator):
    bl_idname = "thirddimensionmedia.addbevel10"
    bl_label = "3DM Add bevel 10mm"
    bl_description = "Add 3DM Bevel with 10mm of width or change the existing 3DM Bevel to 10mm"
    
    def execute(self, context):
        
        selection = bpy.context.selected_objects

        for i in selection:
            
            mod = i.modifiers.get("3DM Bevel")
            if mod is None:
                # otherwise add a modifier to selected object
                mod = i.modifiers.new("3DM Bevel", 'BEVEL')
            i.modifiers["3DM Bevel"].width = 0.01
            i.modifiers["3DM Bevel"].use_clamp_overlap = False
            i.modifiers["3DM Bevel"].offset_type = 'WIDTH'
            i.modifiers["3DM Bevel"].limit_method = 'ANGLE'
                
        return{'FINISHED'}
    
# Add Camera
                
class ThirdDimensionMediaObjectPanel_OT_AddCamera(bpy.types.Operator):
    bl_idname = "thirddimensionmedia.addcamera"
    bl_label = "3DM Add Camera"
    bl_description = "Set this selection in wire mode for viewport and hide the visibily for camera, diffuse, glossy, transmission, volume scatter and shadow. "
    
    def execute(self, context):
        
        bpy.ops.object.camera_add(enter_editmode=False, rotation=(1.5708, 0, 0))
        bpy.context.object.data.sensor_width = 22.3
        bpy.context.object.data.lens = 15
        bpy.context.object.data.clip_end = 800000
        bpy.context.object.data.show_composition_thirds = True
        
        return{'FINISHED'}
            
# 3DM Render Panel
############################################################################################################
        
class ThirdDimensionMediaRenderPanel(bpy.types.Panel):
    bl_label = "3DM Settings"
    bl_idname = "SCENE_PT_RenderPanel"
    bl_space_type = 'PROPERTIES'
    bl_region_type = 'WINDOW'
    bl_context = "render"
    
    def draw(self, context):
        layout = self.layout
        strip = context.scene.render
        strop = context.scene.cycles
        scene = context.scene
        image_settings = strip.image_settings
        
        box = layout.box()
        box.label(text="Camera", icon= 'VIEW_CAMERA')
        row = box.row()
        row.prop(scene, "camera", icon= 'TRIA_RIGHT', text="")
        row = box.row()
        
        box = layout.box()       
        box.label(text="Samples and Dimensions", icon= 'OUTPUT')
        row = box.row()
        row.prop(strop, "samples", text="Render Samples")
        row = box.row()
        row.prop(strip, "resolution_x", text="X")
        row.prop(strip, "resolution_y", text="Y")
        row = box.row()     
        row.prop(strip, "resolution_percentage", text="")
        row = box.row()
        
        box = layout.box()
        box.label(text="Format", icon= 'OUTLINER_OB_IMAGE')
        row = box.row()
        row.prop(scene, "checkboxtransp", text="Transparent Background")
        row = box.row()
        row.label(text="File Format")
        row.prop(scene, "chooseformat", expand=True)
        row = box.row()
        row.scale_y = 1.5     

        box = layout.box()
        box.label(text="Output", icon= 'FILEBROWSER')
        row = box.row()

        row.prop(scene, "checkboxoutput", text="Enable 3DM File Output")
        row = box.row()
        row.prop(strip, "filepath", text="")
        row.operator("thirddimensionmedia.refresh", icon="CHECKMARK", text='')
        row.operator(ThirdDimensionMediaPanel_OT_RangePass2.bl_idname, icon="MOD_MASK", text='')
        row = box.row()
        row.operator(ThirdDimensionMediaPanel_OT_RangePass.bl_idname, text="Render Image", icon='RENDER_STILL')
        row = box.row()
        
# Filepath for 3DM Output
    
class ThirdDimensionMediaPanel_OT_Refresh(bpy.types.Operator):
    bl_idname = "thirddimensionmedia.refresh"
    bl_label = "Valid"
    bl_description = "Valid the path of the 3DM Output"
    
    def execute(self, context):
        
        scene = bpy.context.scene
        compositing_node_tree = scene.node_tree
        filepath = bpy.context.scene.render.filepath
        absolutepath = bpy.path.abspath(filepath)
        
        node = bpy.context.scene.node_tree.nodes.get("3DM_Output", None)
        if node is not None:
            compositing_node_tree.nodes['3DM_Output'].base_path = absolutepath
        else:
            self.report({'INFO'}, "3DM Output is not present")

        return{'FINISHED'}
    
# Set ID to materials and objects
    
class ThirdDimensionMediaPanel_OT_RangePass2(bpy.types.Operator):
    bl_label = "Set ID"
    bl_idname = "pass.rpass2"
    bl_description = "to materials and objects"

    def execute(self, context):
        
        objIndex = 0
        matIndex = 0
     
        for obj in bpy.data.objects:

            obj.pass_index = objIndex + 1
            objIndex += 1
       
        for mat in bpy.data.materials:
       
            mat.pass_index = matIndex + 1
            matIndex += 1
            
        return{'FINISHED'}
    
# Render Button
    
class ThirdDimensionMediaPanel_OT_RangePass(bpy.types.Operator):
    bl_label = "rpass"
    bl_idname = "pass.rpass"
    bl_description = "Valid 3DM Output path, Set ID and Render"

    def execute(self, context):
        
        # Set ID to materials and objects
        objIndex = 0
        matIndex = 0
     
        for obj in bpy.data.objects:

            obj.pass_index = objIndex + 1
            objIndex += 1
       
        for mat in bpy.data.materials:
       
            mat.pass_index = matIndex + 1
            matIndex += 1
          
        # Apply the path to 3DM File Output    
        scene = bpy.context.scene
        compositing_node_tree = scene.node_tree
        filepath = bpy.context.scene.render.filepath
        absolutepath = bpy.path.abspath(filepath)
        
        node = bpy.context.scene.node_tree.nodes.get("3DM_Output", None)
        if node is not None:
            compositing_node_tree.nodes['3DM_Output'].base_path = absolutepath
        else:
            self.report({'INFO'}, "3DM Output is not present")
            
        # Render      
        bpy.ops.render.render("INVOKE_DEFAULT")
            
        return {'FINISHED'}
    
# Enable 3DM Output
        
def enable_3dmoutput(self, context):
        
    scene = bpy.context.scene
    compositing_node_tree = scene.node_tree
        
    if (self.checkboxoutput == True):
        compositing_node_tree.nodes['3DM_Output'].mute = False
    else:
        compositing_node_tree.nodes['3DM_Output'].mute = True
        
                    
    return{'FINISHED'}

# Switch Transparent

def enable_transpa(self, context):
        
    scene = bpy.context.scene
    
        
    if (self.checkboxtransp == True):
        bpy.context.scene.render.film_transparent = True
        bpy.context.scene.cycles.film_transparent_glass = True
        bpy.data.worlds["Real Sky"].node_tree.nodes['3DM_Math'].inputs[1].default_value = 1
        bpy.data.worlds["Hosek-Wilkie"].node_tree.nodes['3DM_Math'].inputs[1].default_value = 1

        
    else:
        bpy.context.scene.render.film_transparent = False
        bpy.context.scene.cycles.film_transparent_glass = False
        bpy.data.worlds["Real Sky"].node_tree.nodes['3DM_Math'].inputs[1].default_value = 0
        bpy.data.worlds["Hosek-Wilkie"].node_tree.nodes['3DM_Math'].inputs[1].default_value = 0
                    
    return{'FINISHED'}

# Switch File Format

def switch_format(self, context):
        
    scene = bpy.context.scene
            
    if (self.chooseformat == 'JPG'):
        bpy.context.scene.render.image_settings.file_format = 'JPEG'
        
    if (self.chooseformat == 'PNG'):
        bpy.context.scene.render.image_settings.file_format = 'PNG'
        bpy.context.scene.render.image_settings.color_mode = 'RGBA'
        
                    
    return{'FINISHED'}

# 3DM Light Panel
############################################################################################################

class ThirdDimensionMediaLightPanel(bpy.types.Panel):
    bl_label = "3DM Light"
    bl_space_type = 'PROPERTIES'
    bl_region_type = 'WINDOW'
    bl_context = "data"
    
    @classmethod
    def poll(cls, context):
        return context.light
    
    def draw(self, context):
        layout = self.layout
        settings = context.scene
        
        row = layout.row()
        col = row.column(align=True)
        col.operator("thirddimensionmedia.setuplight", icon="NODETREE", text='Setup light nodes')
        col.operator("thirddimensionmedia.refreshvalues", icon="FILE_REFRESH", text='Refresh values')
        col = row.column(align=True)
        col.prop(settings, 'blatem')
        col.prop(settings, 'emistre')
        row = layout.row()
        
# Setup Light
                
class ThirdDimensionMediaPanel_OT_SetupLight(bpy.types.Operator):
    bl_idname = "thirddimensionmedia.setuplight"
    bl_label = "3DM Setup Light"
    bl_description = "Enable nodes and add Blackbody"
    
    def execute(self, context):
      
        bpy.context.active_object.data.use_nodes = True
        light   = bpy.context.active_object.data
        nodes   = light.node_tree.nodes
        bpy.context.active_object.data.node_tree.nodes.clear()
        node_output = nodes.new(type='ShaderNodeOutputLight')   
        node_output.location = 200,0
        node_emission = nodes.new(type='ShaderNodeEmission')
        node_emission.location = 0,0
        node_emission.inputs[1].default_value = bpy.context.scene.emistre
        links   = light.node_tree.links
        link = links.new(node_emission.outputs[0], node_output.inputs[0])
        
        node_bb = nodes.new(type="ShaderNodeBlackbody")
        node_bb.inputs[0].default_value = bpy.context.scene.blatem
        node_bb.location = -200,0
        node_ox = nodes.get('Emission')
        links   = light.node_tree.links
        link    = links.new(node_bb.outputs[0], node_ox.inputs[0])

        return{'FINISHED'}
    
# Change Blackbody temperature value
    
def blackbody_temp(self, context):
    settings = context.scene
    
    bpy.context.active_object.data.node_tree.nodes['Blackbody'].inputs[0].default_value = bpy.context.scene.blatem
    
# Change Emission strengh value
    
def emission_stren(self, context):
    settings = context.scene
    
    bpy.context.active_object.data.node_tree.nodes['Emission'].inputs[1].default_value = bpy.context.scene.emistre
                    
    return{'FINISHED'}

# Check and report values from Node Editor

class ThirdDimensionMediaPanel_OT_RefreshValues(bpy.types.Operator):
    bl_idname = "thirddimensionmedia.refreshvalues"
    bl_label = "3DM Refresh"
    bl_description = "Report values from Node Editor"
    
    def execute(self, context):
        
        if (bpy.context.active_object.data.use_nodes == True):
            node = bpy.context.active_object.data.node_tree.nodes.get("Blackbody", None)
            if node is not None:
                bpy.context.scene.blatem = bpy.context.active_object.data.node_tree.nodes['Blackbody'].inputs[0].default_value
                bpy.context.scene.emistre = bpy.context.active_object.data.node_tree.nodes['Emission'].inputs[1].default_value
            else:
                self.report({'INFO'}, "No Blackbody : Please use SETUP LIGHT NODES")
        else:
            self.report({'INFO'}, "Nodes disable : Please use SETUP LIGHT NODES")
            
                    
        return{'FINISHED'}
    
    
# 3DM Material Panel
############################################################################################################

class ThirdDimensionMediaMaterialPanel(bpy.types.Panel):
    bl_label = "3DM Material Setup"
    bl_space_type = 'PROPERTIES'
    bl_region_type = 'WINDOW'
    bl_context = "material"
    
    
    def draw(self, context):
        layout = self.layout
        settings = context.scene
        
        row = layout.row()
        row.operator("thirddimensionmedia.plastic", text='Plastic')
        row.operator("thirddimensionmedia.metal", text='Metal')
        row.operator("thirddimensionmedia.full", text='Full setup')
        row = layout.row()

        
# Setup Plastic
                
class ThirdDimensionMediaPanel_OT_SetupPlastic(bpy.types.Operator):
    bl_idname = "thirddimensionmedia.plastic"
    bl_label = "3DM Setup Plastic Material"
    bl_description = "Create un new material with plastic setup"
    
    def execute(self, context):
      
        activeObject = bpy.context.active_object #Set active object to variable
        mat = bpy.data.materials.new(name="3DM_Plastic") #set new material to variable
        mat.use_nodes = True
        nodes = mat.node_tree.nodes
        
        # Add a diffuse shader and set its location:    
        node = nodes.clear()

        # Add a diffuse shader and set its location:    
        node_output = nodes.new(type='ShaderNodeOutputMaterial')   
        node_output.location = 200,0
        if "3DM_Node_Material_v2" in bpy.data.node_groups:
            node_noodle = nodes.new(type='ShaderNodeGroup')
            node_noodle.node_tree = bpy.data.node_groups['3DM_Node_Material_v2']
            node_noodle.location = 0,0
        else:
            path = os.path.join(os.path.dirname(__file__), "blends/thirdgroups.blend/NodeTree/")
            bpy.ops.wm.append( filename = '3DM_Node_Material_v2', directory = path)
            node_noodle = nodes.new(type='ShaderNodeGroup')
            node_noodle.node_tree = bpy.data.node_groups['3DM_Node_Material_v2']
            node_noodle.location = 0,0
        link = mat.node_tree.links.new(node_noodle.outputs[0], node_output.inputs[0])
        node_noodle.inputs[0].default_value = 0
        node_noodle.inputs[1].default_value = (0.9, 0.9, 0.9, 1)

        activeObject.data.materials.append(mat) #add the material to the object


        return{'FINISHED'}
    
# Setup Plastic 2
                
class ThirdDimensionMediaPanel_OT_SetupPlastic2(bpy.types.Operator):
    bl_idname = "thirddimensionmedia.plastic2"
    bl_label = "3DM Setup Plastic Material"
    bl_description = "Clear the node editor and set a plastic setup"
    
    @classmethod
    def poll(cls, context):
        space = context.space_data

        # Check that it's a ShaderNodeTree, editing 'OBJECT' type and that a node tree (with nodes) exists
        return (space.type == 'NODE_EDITOR') and (space.tree_type == 'ShaderNodeTree') and (space.shader_type == 'OBJECT') and (hasattr(space,"node_tree")) and (hasattr(space.node_tree,"nodes"))
    
    def execute(self, context):
        nodes = bpy.context.object.active_material.node_tree.nodes
        mat = bpy.context.object.active_material
        nodes.clear()
        
        # Add a diffuse shader and set its location:    
        node_output = nodes.new(type='ShaderNodeOutputMaterial')   
        node_output.location = 200,0
        if "3DM_Node_Material_v2" in bpy.data.node_groups:
            node_noodle = nodes.new(type='ShaderNodeGroup')
            node_noodle.node_tree = bpy.data.node_groups['3DM_Node_Material_v2']
            node_noodle.location = 0,0
        else:
            path = os.path.join(os.path.dirname(__file__), "blends/thirdgroups.blend/NodeTree/")
            bpy.ops.wm.append( filename = '3DM_Node_Material_v2', directory = path)
            node_noodle = nodes.new(type='ShaderNodeGroup')
            node_noodle.node_tree = bpy.data.node_groups['3DM_Node_Material_v2']
            node_noodle.location = 0,0
        link = mat.node_tree.links.new(node_noodle.outputs[0], node_output.inputs[0])
        node_noodle.inputs[0].default_value = 0
        node_noodle.inputs[1].default_value = (0.9, 0.9, 0.9, 1)

        return{'FINISHED'}
    
# Setup Metal
                
class ThirdDimensionMediaPanel_OT_SetupMetal(bpy.types.Operator):
    bl_idname = "thirddimensionmedia.metal"
    bl_label = "3DM Setup Plastic Material"
    bl_description = "Create un new material with metal setup"
    
    def execute(self, context):
      
        activeObject = bpy.context.active_object #Set active object to variable
        mat = bpy.data.materials.new(name="3DM_Metal") #set new material to variable
        mat.use_nodes = True
        nodes = mat.node_tree.nodes
        
        # Add a diffuse shader and set its location:    
        node = nodes.clear()

        # Add a diffuse shader and set its location:    
        node_output = nodes.new(type='ShaderNodeOutputMaterial')   
        node_output.location = 200,0
        if "3DM_Node_Material_v2" in bpy.data.node_groups:
            node_noodle = nodes.new(type='ShaderNodeGroup')
            node_noodle.node_tree = bpy.data.node_groups['3DM_Node_Material_v2']
            node_noodle.location = 0,0
        else:
            path = os.path.join(os.path.dirname(__file__), "blends/thirdgroups.blend/NodeTree/")
            bpy.ops.wm.append( filename = '3DM_Node_Material_v2', directory = path)
            node_noodle = nodes.new(type='ShaderNodeGroup')
            node_noodle.node_tree = bpy.data.node_groups['3DM_Node_Material_v2']
            node_noodle.location = 0,0
        if "3DM_Metal_Color" in bpy.data.node_groups:
            node_mcolor = nodes.new(type='ShaderNodeGroup')
            node_mcolor.node_tree = bpy.data.node_groups['3DM_Metal_Color']
            node_mcolor.location = -200,0
        else:
            path = os.path.join(os.path.dirname(__file__), "blends/thirdgroups.blend/NodeTree/")
            bpy.ops.wm.append( filename = '3DM_Metal_Color', directory = path)
            node_mcolor = nodes.new(type='ShaderNodeGroup')
            node_mcolor.node_tree = bpy.data.node_groups['3DM_Metal_Color']
            node_mcolor.location = -200,0
        link = mat.node_tree.links.new(node_noodle.outputs[0], node_output.inputs[0])
        link = mat.node_tree.links.new(node_mcolor.outputs[2], node_noodle.inputs[1])
        node_noodle.inputs[0].default_value = 1
        node_noodle.inputs[7].default_value = 0.4

        activeObject.data.materials.append(mat) #add the material to the object


        return{'FINISHED'}
    
# Setup Metal 2
                
class ThirdDimensionMediaPanel_OT_SetupMetal2(bpy.types.Operator):
    bl_idname = "thirddimensionmedia.metal2"
    bl_label = "3DM Setup Plastic Material"
    bl_description = "Clear the node editor and set a metal setup"
    
    @classmethod
    def poll(cls, context):
        space = context.space_data

        # Check that it's a ShaderNodeTree, editing 'OBJECT' type and that a node tree (with nodes) exists
        return (space.type == 'NODE_EDITOR') and (space.tree_type == 'ShaderNodeTree') and (space.shader_type == 'OBJECT') and (hasattr(space,"node_tree")) and (hasattr(space.node_tree,"nodes"))
    
    def execute(self, context):
        nodes = bpy.context.object.active_material.node_tree.nodes
        mat = bpy.context.object.active_material
        nodes.clear()
        
        # Add a diffuse shader and set its location:    
        node_output = nodes.new(type='ShaderNodeOutputMaterial')   
        node_output.location = 200,0
        if "3DM_Node_Material_v2" in bpy.data.node_groups:
            node_noodle = nodes.new(type='ShaderNodeGroup')
            node_noodle.node_tree = bpy.data.node_groups['3DM_Node_Material_v2']
            node_noodle.location = 0,0
        else:
            path = os.path.join(os.path.dirname(__file__), "blends/thirdgroups.blend/NodeTree/")
            bpy.ops.wm.append( filename = '3DM_Node_Material_v2', directory = path)
            node_noodle = nodes.new(type='ShaderNodeGroup')
            node_noodle.node_tree = bpy.data.node_groups['3DM_Node_Material_v2']
            node_noodle.location = 0,0
        if "3DM_Metal_Color" in bpy.data.node_groups:
            node_mcolor = nodes.new(type='ShaderNodeGroup')
            node_mcolor.node_tree = bpy.data.node_groups['3DM_Metal_Color']
            node_mcolor.location = -200,0
        else:
            path = os.path.join(os.path.dirname(__file__), "blends/thirdgroups.blend/NodeTree/")
            bpy.ops.wm.append( filename = '3DM_Metal_Color', directory = path)
            mcolor = nodes.new(type='ShaderNodeGroup')
            mcolor.node_tree = bpy.data.node_groups['3DM_Metal_Color']
            mcolor.location = -200,0
        link = mat.node_tree.links.new(node_noodle.outputs[0], node_output.inputs[0])
        link = mat.node_tree.links.new(node_mcolor.outputs[2], node_noodle.inputs[1])
        node_noodle.inputs[0].default_value = 1
        node_noodle.inputs[7].default_value = 0.4

        return{'FINISHED'}
    
    
# Setup Full Setup
                
class ThirdDimensionMediaPanel_OT_SetupFull(bpy.types.Operator):
    bl_idname = "thirddimensionmedia.full"
    bl_label = "3DM Setup Full Material"
    bl_description = "Create un new material with full setup"
    
    def execute(self, context):
      
        activeObject = bpy.context.active_object #Set active object to variable
        mat = bpy.data.materials.new(name="3DM_FullSetup") #set new material to variable
        mat.use_nodes = True
        nodes = mat.node_tree.nodes
        
        # Clear the shader editor   
        node = nodes.clear()

        # Add the differents nodes    
        node_output = nodes.new(type='ShaderNodeOutputMaterial')   
        node_output.location = 200,0
        if "3DM_Node_Material_v2" in bpy.data.node_groups:
            node_noodle = nodes.new(type='ShaderNodeGroup')
            node_noodle.node_tree = bpy.data.node_groups['3DM_Node_Material_v2']
            node_noodle.location = 0,0
        else:
            path = os.path.join(os.path.dirname(__file__), "blends/thirdgroups.blend/NodeTree/")
            bpy.ops.wm.append( filename = '3DM_Node_Material_v2', directory = path)
            node_noodle = nodes.new(type='ShaderNodeGroup')
            node_noodle.node_tree = bpy.data.node_groups['3DM_Node_Material_v2']
            node_noodle.location = 0,0
            
        if "3DM_Textures_Preferencies" in bpy.data.node_groups:
            node_prefi = nodes.new(type='ShaderNodeGroup')
            node_prefi.node_tree = bpy.data.node_groups['3DM_Textures_Preferencies']
            node_prefi.location = -500,0
        else:
            path = os.path.join(os.path.dirname(__file__), "blends/thirdgroups.blend/NodeTree/")
            bpy.ops.wm.append( filename = '3DM_Textures_Preferencies', directory = path)
            node_prefi = nodes.new(type='ShaderNodeGroup')
            node_prefi.node_tree = bpy.data.node_groups['3DM_Textures_Preferencies']
            node_prefi.location = -500,0
        
        node_basecolor = nodes.new(type='ShaderNodeTexImage')   
        node_basecolor.location = -300,250
        node_basecolor.label = "Image base color"
        
        node_roughness = nodes.new(type='ShaderNodeTexImage')   
        node_roughness.location = -300,0
        node_roughness.label = "Image roughness"
        
        node_normal = nodes.new(type='ShaderNodeTexImage')   
        node_normal.location = -300,-250
        node_normal.label = "Image normal"
        
        node_bump = nodes.new(type='ShaderNodeTexImage')   
        node_bump.location = -300,-500
        node_bump.label = "Image bump"
        
        #Links between the nodes
        link = mat.node_tree.links.new(node_noodle.outputs[0], node_output.inputs[0])
        link = mat.node_tree.links.new(node_basecolor.outputs[0], node_noodle.inputs[1])
        link = mat.node_tree.links.new(node_roughness.outputs[0], node_noodle.inputs[9])
        link = mat.node_tree.links.new(node_normal.outputs[0], node_noodle.inputs[12])
        link = mat.node_tree.links.new(node_bump.outputs[0], node_noodle.inputs[14])
        link = mat.node_tree.links.new(node_prefi.outputs[0], node_basecolor.inputs[0])
        link = mat.node_tree.links.new(node_prefi.outputs[0], node_roughness.inputs[0])
        link = mat.node_tree.links.new(node_prefi.outputs[0], node_normal.inputs[0])
        link = mat.node_tree.links.new(node_prefi.outputs[0], node_bump.inputs[0])
        
        activeObject.data.materials.append(mat) #add the material to the object

        return{'FINISHED'}
    
# Setup Full Setup 2
                
class ThirdDimensionMediaPanel_OT_SetupFull2(bpy.types.Operator):
    bl_idname = "thirddimensionmedia.full2"
    bl_label = "3DM Setup Full Material"
    bl_description = "Clear the node editor and set a full setup"
    
    @classmethod
    def poll(cls, context):
        space = context.space_data

        # Check that it's a ShaderNodeTree, editing 'OBJECT' type and that a node tree (with nodes) exists
        return (space.type == 'NODE_EDITOR') and (space.tree_type == 'ShaderNodeTree') and (space.shader_type == 'OBJECT') and (hasattr(space,"node_tree")) and (hasattr(space.node_tree,"nodes"))
    
    def execute(self, context):
        
        nodes = bpy.context.object.active_material.node_tree.nodes
        mat = bpy.context.object.active_material
        nodes.clear()

        # Add the differents nodes    
        node_output = nodes.new(type='ShaderNodeOutputMaterial')   
        node_output.location = 200,0
        if "3DM_Node_Material_v2" in bpy.data.node_groups:
            node_noodle = nodes.new(type='ShaderNodeGroup')
            node_noodle.node_tree = bpy.data.node_groups['3DM_Node_Material_v2']
            node_noodle.location = 0,0
        else:
            path = os.path.join(os.path.dirname(__file__), "blends/thirdgroups.blend/NodeTree/")
            bpy.ops.wm.append( filename = '3DM_Node_Material_v2', directory = path)
            node_noodle = nodes.new(type='ShaderNodeGroup')
            node_noodle.node_tree = bpy.data.node_groups['3DM_Node_Material_v2']
            node_noodle.location = 0,0
            
        if "3DM_Textures_Preferencies" in bpy.data.node_groups:
            node_prefi = nodes.new(type='ShaderNodeGroup')
            node_prefi.node_tree = bpy.data.node_groups['3DM_Textures_Preferencies']
            node_prefi.location = -500,0
        else:
            path = os.path.join(os.path.dirname(__file__), "blends/thirdgroups.blend/NodeTree/")
            bpy.ops.wm.append( filename = '3DM_Textures_Preferencies', directory = path)
            node_prefi = nodes.new(type='ShaderNodeGroup')
            node_prefi.node_tree = bpy.data.node_groups['3DM_Textures_Preferencies']
            node_prefi.location = -500,0
        
        node_basecolor = nodes.new(type='ShaderNodeTexImage')   
        node_basecolor.location = -300,250
        node_basecolor.label = "Image base color"
        
        node_roughness = nodes.new(type='ShaderNodeTexImage')   
        node_roughness.location = -300,0
        node_roughness.label = "Image roughness"
        
        node_normal = nodes.new(type='ShaderNodeTexImage')   
        node_normal.location = -300,-250
        node_normal.label = "Image normal"
        
        node_bump = nodes.new(type='ShaderNodeTexImage')   
        node_bump.location = -300,-500
        node_bump.label = "Image bump"
        
        #Links between the nodes
        link = mat.node_tree.links.new(node_noodle.outputs[0], node_output.inputs[0])
        link = mat.node_tree.links.new(node_basecolor.outputs[0], node_noodle.inputs[1])
        link = mat.node_tree.links.new(node_roughness.outputs[0], node_noodle.inputs[9])
        link = mat.node_tree.links.new(node_normal.outputs[0], node_noodle.inputs[12])
        link = mat.node_tree.links.new(node_bump.outputs[0], node_noodle.inputs[14])
        link = mat.node_tree.links.new(node_prefi.outputs[0], node_basecolor.inputs[0])
        link = mat.node_tree.links.new(node_prefi.outputs[0], node_roughness.inputs[0])
        link = mat.node_tree.links.new(node_prefi.outputs[0], node_normal.inputs[0])
        link = mat.node_tree.links.new(node_prefi.outputs[0], node_bump.inputs[0])

        return{'FINISHED'}
        


    
# os path         
############################################################################################################
    
def listfiles(path):
	files = []
	for dirName, subdirList, fileList in os.walk(path):
		# print(dirName)
		# print(subdirList)
		# print(fileList)
		dir = dirName.replace(path, '')
		for fname in fileList:
			files.append(os.path.join(dir, fname))
	return files    

# register     
############################################################################################################

def register():
    bpy.utils.register_class(ThirdDimensionMediaNodesPanel)
    bpy.utils.register_class(ThirdDimensionMediaObjectPanel)
    bpy.utils.register_class(ThirdDimensionMediaRenderPanel)
    bpy.utils.register_class(ThirdDimensionMediaLightPanel)
    bpy.utils.register_class(ThirdDimensionMediaMaterialPanel)
    bpy.utils.register_class(ThirdDimensionMediaPanel_OT_Add3dmMaterial)
    bpy.utils.register_class(ThirdDimensionMediaPanel_OT_Add3dmGlass)
    bpy.utils.register_class(ThirdDimensionMediaPanel_OT_Add3dmLeaf)
    bpy.utils.register_class(ThirdDimensionMediaPanel_OT_Add3dmTprefs)
    bpy.utils.register_class(ThirdDimensionMediaPanel_OT_Add3dmMcolor)
    bpy.utils.register_class(ThirdDimensionMediaPanel_OT_Add3dmPost)
    bpy.utils.register_class(ThirdDimensionMediaPanel_OT_Add3dmDenoiser)
    bpy.utils.register_class(ThirdDimensionMediaPanel_OT_Add3dmZ)
    bpy.utils.register_class(ThirdDimensionMediaPanel_OT_Add3dmMaskID)
    bpy.utils.register_class(ThirdDimensionMediaPanel_OT_Refresh)
    bpy.utils.register_class(ThirdDimensionMediaPanel_OT_RangePass)
    bpy.utils.register_class(ThirdDimensionMediaPanel_OT_RangePass2)
    bpy.utils.register_class(ThirdDimensionMediaPanel_OT_SetupLight)
    bpy.utils.register_class(ThirdDimensionMediaPanel_OT_SetupPlastic)
    bpy.utils.register_class(ThirdDimensionMediaPanel_OT_SetupPlastic2)
    bpy.utils.register_class(ThirdDimensionMediaPanel_OT_SetupMetal)
    bpy.utils.register_class(ThirdDimensionMediaPanel_OT_SetupMetal2)
    bpy.utils.register_class(ThirdDimensionMediaPanel_OT_SetupFull)
    bpy.utils.register_class(ThirdDimensionMediaPanel_OT_SetupFull2)
    bpy.utils.register_class(ThirdDimensionMediaPanel_OT_RefreshValues)
    bpy.utils.register_class(ThirdDimensionMediaObjectPanel_OT_SetItBool)
    bpy.utils.register_class(ThirdDimensionMediaObjectPanel_OT_AddBevel3)
    bpy.utils.register_class(ThirdDimensionMediaObjectPanel_OT_AddBevel5)
    bpy.utils.register_class(ThirdDimensionMediaObjectPanel_OT_AddBevel10)
    bpy.utils.register_class(ThirdDimensionMediaObjectPanel_OT_AddCamera)
    bpy.utils.register_class(ThirdDimensionMediaMenu)
    bpy.types.NODE_MT_add.append(menu_draw)

    
    bpy.types.Scene.checkboxoutput = bpy.props.BoolProperty(
        name = "Enable 3DM Output",
        description = "Enable 3DM Output",
        default = False,
        update = enable_3dmoutput
        )
    
    bpy.types.Scene.checkboxtransp = bpy.props.BoolProperty(
        name = "Enable Transparent",
        description = "Enable Transparent",
        default = False,
        update = enable_transpa
        )
        
    bpy.types.Scene.chooseformat = bpy.props.EnumProperty(
        name = "Switch Image Format",
        items = [
            ('JPG','JPG',''),
            ('PNG','PNG', '')],
        description = "Switch Image Format",
        default = "PNG",
        update = switch_format
        )
        
    bpy.types.Scene.blatem = bpy.props.FloatProperty(
        name = "Temperature",
        description = "Change the light's temperature",
        min = 1000,
        max = 12000,
        step = 10000,
        precision = 0,
        default = 5000,
        update = blackbody_temp
        )
        
    bpy.types.Scene.emistre = bpy.props.FloatProperty(
        name = "Strengh",
        description = "Change the light's strengh",
        min = 0,
        max = 50000,
        step = 10,
        precision = 1,
        default = 1,
        update = emission_stren
        )


def unregister():
    bpy.utils.unregister_class(ThirdDimensionMediaNodesPanel)
    bpy.utils.unregister_class(ThirdDimensionMediaObjectPanel)
    bpy.utils.unregister_class(ThirdDimensionMediaRenderPanel)
    bpy.utils.unregister_class(ThirdDimensionMediaLightPanel)
    bpy.utils.unregister_class(ThirdDimensionMediaMaterialPanel)
    bpy.utils.unregister_class(ThirdDimensionMediaPanel_OT_Add3dmMaterial)
    bpy.utils.unregister_class(ThirdDimensionMediaPanel_OT_Add3dmGlass)
    bpy.utils.unregister_class(ThirdDimensionMediaPanel_OT_Add3dmLeaf)
    bpy.utils.unregister_class(ThirdDimensionMediaPanel_OT_Add3dmTprefs)
    bpy.utils.unregister_class(ThirdDimensionMediaPanel_OT_Add3dmMcolor)
    bpy.utils.unregister_class(ThirdDimensionMediaPanel_OT_Add3dmPost)
    bpy.utils.unregister_class(ThirdDimensionMediaPanel_OT_Add3dmDenoiser)
    bpy.utils.unregister_class(ThirdDimensionMediaPanel_OT_Add3dmZ)
    bpy.utils.unregister_class(ThirdDimensionMediaPanel_OT_Add3dmMaskID)
    bpy.utils.unregister_class(ThirdDimensionMediaPanel_OT_Refresh)
    bpy.utils.unregister_class(ThirdDimensionMediaPanel_OT_RangePass)
    bpy.utils.unregister_class(ThirdDimensionMediaPanel_OT_RangePass2)
    bpy.utils.unregister_class(ThirdDimensionMediaPanel_OT_SetupLight)
    bpy.utils.unregister_class(ThirdDimensionMediaPanel_OT_SetupPlastic)
    bpy.utils.unregister_class(ThirdDimensionMediaPanel_OT_SetupPlastic2)
    bpy.utils.unregister_class(ThirdDimensionMediaPanel_OT_SetupMetal)
    bpy.utils.unregister_class(ThirdDimensionMediaPanel_OT_SetupMetal2)
    bpy.utils.unregister_class(ThirdDimensionMediaPanel_OT_SetupFull)
    bpy.utils.unregister_class(ThirdDimensionMediaPanel_OT_SetupFull2)
    bpy.utils.unregister_class(ThirdDimensionMediaPanel_OT_RefreshValues)
    bpy.utils.unregister_class(ThirdDimensionMediaObjectPanel_OT_SetItBool)
    bpy.utils.unregister_class(ThirdDimensionMediaObjectPanel_OT_AddBevel3)
    bpy.utils.unregister_class(ThirdDimensionMediaObjectPanel_OT_AddBevel5)
    bpy.utils.unregister_class(ThirdDimensionMediaObjectPanel_OT_AddBevel10)
    bpy.utils.unregister_class(ThirdDimensionMediaObjectPanel_OT_AddCamera)
    bpy.utils.unregister_class(ThirdDimensionMediaMenu)
    bpy.types.NODE_MT_add.remove(menu_draw)
    
    del bpy.types.Scene.checkboxoutput
    del bpy.types.Scene.checkboxtransp
    del bpy.types.Scene.chooseformat
    del bpy.types.Scene.blatem
    del bpy.types.Scene.emistre


if __name__ == "__main__":
    register()